package com.isu.gaswellwatch.modbus.data.decode.listener;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 0:32
 */

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;


@Component
public class DynamicRabbitListener implements ApplicationRunner {

    private final AmqpAdmin amqpAdmin;
    private final ComposeModbusMessageListener composeListener;
    private final SimpleMessageListenerContainer listenerContainer;

    public DynamicRabbitListener(ConnectionFactory connectionFactory) {
        this.amqpAdmin = new RabbitAdmin(connectionFactory);
        this.listenerContainer = new SimpleMessageListenerContainer();
        this.listenerContainer.setConnectionFactory(connectionFactory);
        this.listenerContainer.setMessageListener(this.composeListener = new ComposeModbusMessageListener());
        // 启动监听容器
        this.listenerContainer.start();
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        this.composeListener.addBatchMessageListener(new ModbusMessageBackupListener());
        this.composeListener.addBatchMessageListener(new ModbusMessagePersistListener());
        // TODO 根据设备自动绑定队列
        this.addListenerQueue("/modbus/device/4B454E454E4731343030303030333538/collect");
    }

//    @RabbitListener(queues = "/modbus/device/4B454E454E4731343030303030333538/collect")
//    public void modbusMessage(Message message) {
//        System.out.println("modbusMessage:" + new String(message.getBody()));
//    }

    public void addListenerQueue(String queueName) {
        this.addListenerQueue(queueName, null, null);
    }

    public void addListenerQueue(String queueName, String exchangeName, String routingKey) {
        Queue queue = QueueBuilder.durable(queueName).build();
        // 声明队列
        this.amqpAdmin.declareQueue(queue);

        if (StringUtils.isNotBlank(exchangeName) && StringUtils.isNotBlank(routingKey)) {
            // 声明直接类型的交换器
            DirectExchange exchange = new DirectExchange(exchangeName);
            this.amqpAdmin.declareExchange(exchange);
            // 绑定队列和交换器
            this.amqpAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(routingKey));
        }

        // 设置监听的队列
        this.listenerContainer.addQueues(queue);
    }

}